import curlify
import requests
from requests.adapters import HTTPAdapter

from reach_commons.app_logging.logger import get_reach_logger


class HttpRequestLogger(HTTPAdapter):
    """
    A custom HTTPAdapter for logging details of HTTP requests and responses.
    This class logs each request and response including URL, method, headers,
    and body. It also includes a unique UUID for each request, along with
    business_id and pos_partner_name for easier query and identification.

    Attributes:
        business_id (str): Identifier for the business.
        pos_partner_name (str): Name of the POS partner.

    Example of use:
        # Create a logging session with business details
        session = create_logging_session(business_id="12345",
                                         pos_partner_name="ExamplePOS")

        # Use the session to make requests
        response = session.get("https://httpbin.org/get", params={"test": "value"})
        # Request and response details will be logged with UUID, business_id, and pos_partner_name
    """

    def __init__(
        self, logger=get_reach_logger(), business_id=None, pos_partner_name=None
    ):
        super().__init__()
        self.logger = logger
        self.business_id = business_id
        self.pos_partner_name = pos_partner_name

    def send(self, request, **kwargs):
        # Unique identifier for each request
        self.logger.info(f"URL: {request.url}")
        self.logger.info(f"Method: {request.method}")
        self.logger.info(f"Headers: {request.headers}")
        self.logger.info(f"Body: {request.body}")

        # Send the request
        response = super(HttpRequestLogger, self).send(request, **kwargs)

        # Log response details with business_id, pos_partner_name, and request_uuid
        self.logger.info(f"Response Status: {response.status_code}")
        self.logger.info(f"Response Headers: {response.headers}")
        self.logger.info(f"Response Content: {response.content}")
        self.logger.info(f"Complete Curl: {curlify.to_curl(response.request)}")

        return response


def create_logging_session(
    logger=get_reach_logger(), business_id=None, pos_partner_name=None
):
    session = requests.Session()
    logger_adapter = HttpRequestLogger(
        logger=logger, business_id=business_id, pos_partner_name=pos_partner_name
    )
    session.mount("http://", logger_adapter)
    session.mount("https://", logger_adapter)
    return session



import os

import requests


class ReachDataBridgeClient:
    def __init__(self, environment=None):
        self.environment = environment or os.environ.get("ENV", "Staging")

    @property
    def base_url(self):
        return {
            "Staging": "https://api-staging.getreach.ai",
            "Prod": "https://api.getreach.ai",
        }.get(self.environment)

    @property
    def gateway_headers(self):
        common_headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }

        auth_token = {
            "Staging": "Bearer 6c6adc18-22a9-929f-648d-786eb20ebcf4",
            "Prod": "Bearer a57a82c0-0493-563b-ba1d-6c63c201ce20",
        }.get(self.environment)

        return {**common_headers, "Authorization": auth_token}

    def stripe_create_subscription(self, user_id, business_id):
        resp = requests.post(
            f"{self.base_url}/stripe/user/{user_id}/business/{business_id}/subscription",
            headers=self.gateway_headers,
        )
        return resp

    def stripe_create_customer(self, user_id):
        resp = requests.post(
            f"{self.base_url}/stripe/user/{user_id}/customer",
            headers=self.gateway_headers,
        )
        return resp

    def stripe_create_booking_guarantee(self, business_id, booking_price):
        resp = requests.post(
            f"{self.base_url}/stripe/business/{business_id}/booking-guarantee/",
            headers=self.gateway_headers,
            params={"booking_price": booking_price},
        )
        return resp

    def stripe_get_customer(self, user_id):
        resp = requests.get(
            f"{self.base_url}/stripe/user/{user_id}/customer",
            headers=self.gateway_headers,
        )
        return resp

    def stripe_get_cards(self, user_id):
        resp = requests.get(
            f"{self.base_url}/stripe/user/{user_id}/cards", headers=self.gateway_headers
        )
        return resp

    def stripe_set_default_payment_method(self, user_id: int, default_payment_method):
        resp = requests.patch(
            f"{self.base_url}/stripe/user/{user_id}/default_payment/{default_payment_method}",
            headers=self.gateway_headers,
        )
        return resp

    def stripe_create_setup_intent(self, user_id: int):
        resp = requests.post(
            f"{self.base_url}/stripe/user/{user_id}/setup_intent",
            headers=self.gateway_headers,
        )
        return resp

    def stripe_delete_payment_method(self, payment_method: str):
        resp = requests.delete(
            f"{self.base_url}/stripe/payment_method/{payment_method}",
            headers=self.gateway_headers,
        )
        return resp

    def stripe_cancel_subscription(self, business_id: int):
        resp = requests.delete(
            f"{self.base_url}/stripe/business/{business_id}",
            headers=self.gateway_headers,
        )
        return resp

    def twilio_create_business_phone_number(self, business_id=None):
        resp = requests.post(
            f"{self.base_url}/twilio/create-business-phone-number",
            headers=self.gateway_headers,
            params={
                "business_id": business_id,
            },
        )
        return resp

    def twilio_remove_phone_number(
        self, business_id=None, phone_number=None, remove_from_db=False
    ):
        resp = requests.delete(
            f"{self.base_url}/twilio/remove-phone-number",
            headers=self.gateway_headers,
            params={
                "business_id": business_id,
                "phone_number": phone_number,
                "remove_from_db": remove_from_db,
            },
        )
        return resp

    def twilio_manage_toll_free_verification_review(
        self, business_id=None, phone_number=None
    ):
        resp = requests.post(
            f"{self.base_url}/twilio/manage-toll-free-verification-review",
            headers=self.gateway_headers,
            params={
                "business_id": business_id,
                "phone_number": phone_number,
            },
        )
        return resp

    def twilio_business_phone_details(self, business_id=None, phone_number=None):
        resp = requests.get(
            f"{self.base_url}/twilio/business-phone-details",
            headers=self.gateway_headers,
            params={
                "business_id": business_id,
                "phone_number": phone_number,
            },
        )
        return resp

    def twilio_move_phone_number_to_subaccount(self, business_id=None):
        resp = requests.post(
            f"{self.base_url}/twilio/{business_id}/migrate-to-subaccount",
            headers=self.gateway_headers,
        )
        return resp

    def stripe_subscription_one_time_recharge(
        self, business_id=None, price_id=None, charge_strategy=None, refill_txn_id=None
    ):
        resp = requests.post(
            f"{self.base_url}/stripe/subscription/{business_id}/one-time-charge/{price_id}",
            headers=self.gateway_headers,
            params={
                "charge_strategy": charge_strategy,
                "refill_txn_id": refill_txn_id,
            },
        )
        return resp


#
# api_gateway = ReachApiGatewayV2()
# stripe_customer = api_gateway.stripe_create_customer(
#     user_id=1,
#     description="wilson",
#     name="teste",
#     email="teste@gmail.com",
#     phone=None,
# )



import logging
import os


def init_logger(name: str):
    logging.basicConfig(
        level=getattr(logging, os.getenv("LOG_LEVEL", "INFO").upper(), logging.INFO),
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        force=True,
    )
    for noisy in ("botocore", "boto3", "urllib3"):
        logging.getLogger(noisy).setLevel(logging.WARNING)
    return logging.getLogger(name)


def log_with_event(level_fn, msg, event):
    level_fn(f"{msg} | event={event}")



# mongo_token_bucket.py
import random
import time
from dataclasses import dataclass
from typing import Optional, Tuple

_LUA_WINDOW_LIMITER = """
-- KEYS[1] = window key
-- ARGV[1] = tokens_to_consume
-- ARGV[2] = ttl_seconds
-- ARGV[3] = limit

local tokens = tonumber(ARGV[1])
local ttl = tonumber(ARGV[2])
local limit = tonumber(ARGV[3])

local current = redis.call('INCRBY', KEYS[1], tokens)

-- If this was the first increment (key was 0 / nonexistent), set TTL
if current == tokens then
  redis.call('EXPIRE', KEYS[1], ttl)
end

if current <= limit then
  return 1
else
  return 0
end
"""


@dataclass(frozen=True)
class AcquireResult:
    allowed: bool
    retry_after_seconds: int  # for visibility timeout / delay


class MongoTokenBucketManager:
    """
    Window-based limiter:
      - limit tokens per interval_seconds
      - atomic via Redis Lua
    """

    def __init__(
        self,
        redis_manager,
        limit_per_window: int,
        interval_seconds: int = 2,
        bucket_key: str = "global",
        jitter_seconds: Optional[int] = None,
        key_prefix: str = "mongo_write_budget",
    ):
        if interval_seconds <= 0:
            raise ValueError("interval_seconds must be > 0")
        if limit_per_window <= 0:
            raise ValueError("limit_per_window must be > 0")

        self.redis = redis_manager
        self.limit = int(limit_per_window)
        self.interval = int(interval_seconds)
        self.bucket_key = bucket_key
        self.key_prefix = key_prefix
        self.jitter = (
            int(jitter_seconds) if jitter_seconds is not None else self.interval
        )  # default: 0..interval

        # Cache the script SHA if you want; eval is fine for now (2-day fix).
        self._lua = _LUA_WINDOW_LIMITER

    def _now(self) -> float:
        return time.time()

    def _window_start(self, now: float) -> int:
        return int(now // self.interval) * self.interval

    def _redis_key(self, window_start: int) -> str:
        return f"{self.key_prefix}:{self.bucket_key}:{window_start}"

    def acquire(self, tokens: int = 1) -> AcquireResult:
        """
        Try to consume tokens. If denied, returns retry_after_seconds to push visibility timeout.
        """
        now = self._now()
        window_start = self._window_start(now)
        window_end = window_start + self.interval

        key = self._redis_key(window_start)

        # TTL a bit bigger than the window so old keys go away safely
        ttl_seconds = max(self.interval * 2, 5)

        allowed = self.redis.eval(
            self._lua,
            numkeys=1,
            keys=[key],
            args=[str(int(tokens)), str(int(ttl_seconds)), str(int(self.limit))],
        )

        if allowed == 1:
            return AcquireResult(allowed=True, retry_after_seconds=0)

        # Denied: retry after next window, plus jitter to avoid waves
        base = max(0.0, window_end - now)  # seconds until next window
        jitter = random.uniform(0.0, float(self.jitter))
        retry_after = int(max(1.0, base + jitter))  # at least 1s

        return AcquireResult(allowed=False, retry_after_seconds=retry_after)



import json
import os

import requests


class ReachOpsApiClient:
    def __init__(self, environment=None):
        self.environment = environment or os.environ.get("ENV", "Staging")

    @property
    def base_url(self):
        return {
            "Staging": "https://api-staging.getreach.ai/ops/v1",
            "Prod": "https://api.getreach.ai/ops/v1",
        }.get(self.environment)

    @property
    def gateway_headers(self):
        common_headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }

        auth_token = {
            "Staging": "Bearer 6c6adc18-22a9-929f-648d-786eb20ebcf4",
            "Prod": "Bearer a57a82c0-0493-563b-ba1d-6c63c201ce20",
        }.get(self.environment)

        return {**common_headers, "Authorization": auth_token}

    def business_google_review_aggregated_by_month_recalculate(
        self, business_id, business_joined_at=None
    ):
        resp = requests.post(
            f"{self.base_url}/businesses/{business_id}/google-reviews/aggregated-by-month/recalculate",
            headers=self.gateway_headers,
            params={"business_joined_at": business_joined_at},
        )
        return resp

    def business_google_review_active_sync(
        self, business_id, place_id=None, joined_at=None
    ):
        resp = requests.post(
            f"{self.base_url}/businesses/{business_id}/google-reviews/activate-sync",
            headers=self.gateway_headers,
            data=json.dumps({"place_id": place_id, "joined_at": joined_at}),
        )
        return resp

    def business_google_review_initiate_resync(self, business_id):
        resp = requests.post(
            f"{self.base_url}/businesses/{business_id}/google-reviews/initiate-resync",
            headers=self.gateway_headers,
        )
        return resp

    def conversations_balances_apply_credits_from_invoice(self, invoice: dict):
        resp = requests.post(
            f"{self.base_url}/conversations/balances/apply-credits-from-invoice",
            headers=self.gateway_headers,
            data=json.dumps(invoice),
        )
        return resp

    def enable_text_ai(self, business_id):
        resp = requests.post(
            f"{self.base_url}/conversations/{business_id}/text-ai/enable",
            headers=self.gateway_headers,
        )
        return resp
